/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "LMI_IPConfigurationService.h"
#include <LMI_IPProtocolEndpoint.h>
#include <LMI_IPAssignmentSettingData.h>
#include "network.h"
#include "port.h"
#include "endpoint.h"
#include "connection.h"
#include "setting.h"

static const CMPIBroker* _cb = NULL;

static void LMI_IPConfigurationServiceInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_IPConfigurationServiceCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IPConfigurationServiceEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_IPConfigurationServiceEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    const char *ns = KNameSpace(cop);

    LMI_IPConfigurationService w;
    LMI_IPConfigurationService_Init(&w, _cb, ns);
    LMI_IPConfigurationService_Set_SystemName(&w, get_system_name());
    LMI_IPConfigurationService_Set_SystemCreationClassName(&w, get_system_creationg_class_name());
    LMI_IPConfigurationService_Set_CreationClassName(&w, "LMI_IPConfigurationService");
    LMI_IPConfigurationService_Set_Name(&w, "IPConfigurationService");
    KReturnInstance(cr, w);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IPConfigurationServiceGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_IPConfigurationServiceCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_IPConfigurationServiceModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_IPConfigurationServiceDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_IPConfigurationServiceExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_IPConfigurationService,
    LMI_IPConfigurationService,
    _cb,
    LMI_IPConfigurationServiceInitialize(&mi))

static void LMI_IPConfigurationServiceMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_IPConfigurationServiceMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IPConfigurationServiceInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return LMI_IPConfigurationService_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_IPConfigurationService,
    LMI_IPConfigurationService,
    _cb,
    LMI_IPConfigurationServiceMethodInitialize(&mi))

KUint32 LMI_IPConfigurationService_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_IPConfigurationService_ChangeAffectedElementsAssignedSequence(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KRefA* ManagedElements,
    const KUint16A* AssignedSequence,
    KRef* Job,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_IPConfigurationService_AddStaticIPv4Interface(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KRef* Configuration,
    KRef* StaticSetting,
    const KString* Address,
    const KString* SubnetMask,
    const KString* Gateway,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_IPConfigurationService_ApplySettingToLANEndpoint(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KRef* ConfigurationRef,
    const KRef* EndpointRef,
    KRef* Job,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_IPConfigurationService_ApplySettingToIPNetworkConnection(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KRef* SettingData,
    const KRef* IPVersionSettingData,
    const KRef* IPNetworkConnection,
    const KUint16* Mode,
    KRef* Job,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_IPConfigurationService_StartService(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_IPConfigurationService_StopService(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_IPConfigurationService_ApplySettingToIPProtocolEndpoint(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KRef* ConfigurationRef,
    const KRef* EndpointRef,
    KRef* Job,
    CMPIStatus* status)
{
    Network *network = mi->hdl;
    size_t i, j;
    KUint32 result = KUINT32_INIT;

    Require(ConfigurationRef, "No configuration has been specified");
    Require(EndpointRef, "No endpoint has been specified");

    LMI_IPProtocolEndpointRef protocolEndpointRef;
    LMI_IPProtocolEndpointRef_InitFromObjectPath(&protocolEndpointRef, _cb, EndpointRef->value);
    if (strcmp(protocolEndpointRef.SystemName.chars, get_system_name()) != 0) {
        KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "Endpoint doesn't exists");
        return result;
    }
    network_lock(network);
    const Ports *ports = network_get_ports(network);
    const Endpoints *endpoints;
    Endpoint *endpoint = NULL;
    for (i = 0; i < ports_length(ports); ++i) {
        endpoints = network_get_endpoints_by_port(network, ports_index(ports, i));
        for (j = 0; j < endpoints_length(endpoints); ++j) {
            if (strcmp(endpoint_get_name(endpoints_index(endpoints, j)), protocolEndpointRef.Name.chars) == 0) {
                endpoint = endpoints_index(endpoints, j);
                break;
            }
        }
    }

    LMI_IPAssignmentSettingData settingDataRef;
    LMI_IPAssignmentSettingData_InitFromObjectPath(&settingDataRef, _cb, ConfigurationRef->value);
    const char *id = settingDataRef.InstanceID.chars;

    const Connections *connections = network_get_connections(network);
    Connection *connection = NULL;
    for (i = 0; i < connections_length(connections); ++i) {
        if (strcmp(connection_get_id(connections_index(connections, i)), id) == 0) {
            connection = connections_index(connections, i);
        }
    }

    if (endpoint == NULL) {
        KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "Endpoint doesn't exists");
        network_unlock(network);
        return result;
    }

    if (connection == NULL) {
        KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "Connection doesn't exists");
        network_unlock(network);
        return result;
    }

    if ((result.value = network_activate_connection(network, endpoint_get_port(endpoint), connection)) != 0) {
        warn("Unable to activate network connection (%d)", result.value);
        KSetStatus2(_cb, status, ERR_FAILED, "Unable to activate network connection");
        network_unlock(network);
        return result;
    }

    KSetStatus(status, OK);
    KUint32_Set(&result, 0);
    network_unlock(network);
    return result;
}

KUint32 LMI_IPConfigurationService_ApplySettingToComputerSystem(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KRef* IPVersionSettingData,
    const KRef* ComputerSystem,
    const KUint16* Mode,
    KRef* Job,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_IPConfigurationService_CreateStaticSetting(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KString* Caption,
    const KUint16* ProtocolIFType,

    const KString* IPv4Address,
    const KString* IPv4SubnetMask,
    const KString* IPv4DefaultGateway,
    const KStringA* IPv4DNSServers,
    const KStringA* IPv4SearchDomains,

    const KString* IPv6Address,
    const KUint8* IPv6PrefixLength,
    const KString* IPv6DefaultGateway,
    const KStringA* IPv6DNSServers,
    const KStringA* IPv6SearchDomains,

    const KRef* Port,

    KRef* SettingData,

    CMPIStatus* status)
{
    size_t i, count;
    Network *network = mi->hdl;
    KUint32 result = KUINT32_INIT;

    Require(Caption, "Caption hasn't been specified");
    Require(ProtocolIFType, "Protocol type hasn't been specified");
    if (ProtocolIFType->value == 0) {
        KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "ProtocolIFType can't be 0");
        return result;
    }

    network_lock(network);
    Connection *connection = connection_new(network, NULL, strdup(Caption->chars));
    if (ProtocolIFType->value == 1 || ProtocolIFType->value == 3) {
        // IPv4
        Require(IPv4Address, "IPv4 address hasn't been specified");
        Require(IPv4SubnetMask, "IPv4 subnet mask hasn't been specified");

        // Gateway is not required
        const char *gateway;
        if (!IPv4DefaultGateway->null && IPv4DefaultGateway->exists && strlen(IPv4DefaultGateway->chars) > 0) {
            gateway = IPv4DefaultGateway->chars;
        } else {
            gateway = NULL;
        }

        Setting *setting = setting_new_init(IPv4, SETTING_STATIC, IPv4Address->chars, netmaskToPrefix4(IPv4SubnetMask->chars), gateway);

        if (!IPv4DNSServers->null && IPv4DNSServers->exists) {
            // TODO: poke konkret upstream to fix usage of IPv4DNSServers->count
            count = IPv4DNSServers->value->ft->getSize(IPv4DNSServers->value, NULL);
            for (i = 0; i < count; ++i) {
                // TODO: poke konkret upstream to constify KStringA_Get
                debug("DNS: %s", KStringA_Get((KStringA *) IPv4DNSServers, i));
                setting_add_dns_server(setting, KStringA_Get((KStringA *) IPv4DNSServers, i));
            }
        }
        if (!IPv4SearchDomains->null && IPv4SearchDomains->exists) {
            count = IPv4DNSServers->value->ft->getSize(IPv4SearchDomains->value, NULL);
            for (i = 0; i < count; ++i) {
                // TODO: poke konkret upstream to constify KStringA_Get
                setting_add_search_domain(setting, KStringA_Get((KStringA *) IPv4SearchDomains, i));
            }
        }

        connection_add_setting(connection, setting);
    }
    if (ProtocolIFType->value == 2 || ProtocolIFType->value == 3) {
        // IPv6
        Require(IPv6Address, "IPv6 address hasn't been specified");
        Require(IPv6PrefixLength, "IPv6 prefix length hasn't been specified");

        // Gateway is not required
        const char *gateway;
        if (!IPv6DefaultGateway->null && IPv6DefaultGateway->exists && strlen(IPv6DefaultGateway->chars) > 0) {
            gateway = IPv6DefaultGateway->chars;
        } else {
            gateway = NULL;
        }

        Setting *setting = setting_new_init(IPv6, SETTING_STATIC, IPv6Address->chars, IPv6PrefixLength->value, gateway);
        if (!IPv6DNSServers->null && IPv6DNSServers->exists) {
            count = IPv6DNSServers->value->ft->getSize(IPv6DNSServers->value, NULL);
            for (i = 0; i < count; ++i) {
                // TODO: poke konkret upstream to constify KStringA_Get
                debug("DNS: %s", KStringA_Get((KStringA *) IPv6DNSServers, i));
                setting_add_dns_server(setting, KStringA_Get((KStringA *) IPv6DNSServers, i));
            }
        }
        if (!IPv6SearchDomains->null && IPv6SearchDomains->exists) {
            count = IPv6SearchDomains->value->ft->getSize(IPv6SearchDomains->value, NULL);
            for (i = 0; i < count; ++i) {
                // TODO: poke konkret upstream to constify KStringA_Get
                setting_add_search_domain(setting, KStringA_Get((KStringA *) IPv6SearchDomains, i));
            }
        }

        connection_add_setting(connection, setting);
    }

    if (!Port->null && Port->exists) {
        const Ports *ports = network_get_ports(network);
        CIM_NetworkPort networkPort;
        CIM_NetworkPort_InitFromObjectPath(&networkPort, _cb, Port->value);
        const char *name = networkPort.DeviceID.chars;
        const char *port_name;
        for (size_t i = 0; i < ports_length(ports); ++i) {
            port_name = port_get_id(ports_index(ports, i));
            if (strcmp(port_name, name) == 0) {
                connection_set_port(connection, ports_index(ports, i));
                break;
            }
        }
    }

    if (network_create_connection(network, connection) != 0) {
        KSetStatus2(_cb, status, ERR_FAILED, "Unable to create network connection");
        network_unlock(network);
        return result;
    }

    // Return reference to LMI_IPAssignmentSettingData
    const char *ns = KNameSpace(LMI_IPConfigurationServiceRef_ToObjectPath(self, NULL));
    LMI_IPAssignmentSettingDataRef ref;
    LMI_IPAssignmentSettingDataRef_Init(&ref, _cb, ns);
    LMI_IPAssignmentSettingDataRef_Set_InstanceID(&ref, connection_get_id(connection));

    KRef_SetObjectPath(SettingData, LMI_IPAssignmentSettingDataRef_ToObjectPath(&ref, NULL));

    KSetStatus(status, OK);
    KUint32_Set(&result, 0);
    network_unlock(network);
    return result;
}

KUint32 LMI_IPConfigurationService_CreateDHCPSetting(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPConfigurationServiceRef* self,
    const KString* Caption,
    const KUint16* ProtocolIFType,
    const KRef* Port,
    const KString* ClientIdentification,

    KRef* SettingData,

    CMPIStatus* status)
{
    Network *network = mi->hdl;
    KUint32 result = KUINT32_INIT;

    Require(Caption, "Caption hasn't been specified");
    Require(ProtocolIFType, "Protocol type hasn't been specified");
    if (ProtocolIFType->value == 0) {
        KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "ProtocolIFType can't be 0");
        return result;
    }

    network_lock(network);
    Connection *connection = connection_new(network, NULL, strdup(Caption->chars));
    if (ProtocolIFType->value == 1 || ProtocolIFType->value == 3) {
        // IPv4
        Setting *setting = setting_new(IPv4, SETTING_DHCP);
        if (!ClientIdentification->null && ClientIdentification->exists) {
            setting_set_clientID(setting, ClientIdentification->chars);
        }
        connection_add_setting(connection, setting);
    }
    if (ProtocolIFType->value == 2 || ProtocolIFType->value == 3) {
        // IPv6
        Setting *setting = setting_new(IPv6, SETTING_DHCP);
        if (!ClientIdentification->null && ClientIdentification->exists) {
            setting_set_clientID(setting, ClientIdentification->chars);
        }
        connection_add_setting(connection, setting);
    }

    if (!Port->null && Port->exists) {
        const Ports *ports = network_get_ports(network);
        CIM_NetworkPort networkPort;
        CIM_NetworkPort_InitFromObjectPath(&networkPort, _cb, Port->value);
        const char *name = networkPort.DeviceID.chars;
        const char *port_name;
        for (size_t i = 0; i < ports_length(ports); ++i) {
            port_name = port_get_id(ports_index(ports, i));
            if (strcmp(port_name, name) == 0) {
                connection_set_port(connection, ports_index(ports, i));
                break;
            }
        }
    }

    if (network_create_connection(network, connection) != 0) {
        KSetStatus2(_cb, status, ERR_FAILED, "Unable to create network connection");
        network_unlock(network);
        return result;
    }

    // Return reference to LMI_IPAssignmentSettingData
    const char *ns = KNameSpace(LMI_IPConfigurationServiceRef_ToObjectPath(self, NULL));
    LMI_IPAssignmentSettingDataRef ref;
    LMI_IPAssignmentSettingDataRef_Init(&ref, _cb, ns);
    LMI_IPAssignmentSettingDataRef_Set_InstanceID(&ref, connection_get_id(connection));

    KRef_SetObjectPath(SettingData, LMI_IPAssignmentSettingDataRef_ToObjectPath(&ref, NULL));

    KSetStatus(status, OK);
    KUint32_Set(&result, 0);
    network_unlock(network);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_IPConfigurationService",
    "LMI_IPConfigurationService",
    "instance method")
