/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "endpoint.h"

#include "endpoint_private.h"

Endpoint *endpoint_new(Port *port)
{
    Endpoint *e = malloc(sizeof(Endpoint));
    e->ipv4address = NULL;
    e->ipv4netmask = NULL;
    e->ipv4gateway = NULL;
    e->ipv6address = NULL;
    e->ipv6gateway = NULL;
    e->name = NULL;
    e->port = port;
    return e;
}

ProtocolType endpoint_get_type(const Endpoint *endpoint)
{
    return endpoint->type;
}

const char *endpoint_get_name(const Endpoint *endpoint)
{
    return endpoint->name;
}

Port *endpoint_get_port(const Endpoint *endpoint)
{
    return endpoint->port;
}

const char *endpoint_get_ipv4address(const Endpoint *endpoint)
{
    return endpoint->ipv4address;
}

const char *endpoint_get_ipv4netmask(const Endpoint *endpoint)
{
    return endpoint->ipv4netmask;
}

const char *endpoint_get_ipv4gateway(const Endpoint *endpoint)
{
    return endpoint->ipv4gateway;
}

const char *endpoint_get_ipv6address(const Endpoint *endpoint)
{
    return endpoint->ipv6address;
}

uint8_t endpoint_get_ipv6prefix(const Endpoint *endpoint)
{
    return endpoint->ipv6prefix;
}

const char *endpoint_get_ipv6gateway(const Endpoint *endpoint)
{
    return endpoint->ipv6gateway;
}

void endpoint_free(Endpoint *e)
{
    free(e->ipv4address);
    free(e->ipv4netmask);
    free(e->ipv4gateway);
    free(e->ipv6address);
    free(e->ipv6gateway);
    free(e->name);
    free(e);
}

LIST_IMPL(Endpoint, endpoint)
