/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#ifndef ENDPOINT_H
#define ENDPOINT_H

#include "globals.h"

/**
 * Create new endpoint that belongs to the port \p port.
 *
 * \param port \p Port to which the endpoint belongs
 * \return \p Endpoint
 */
Endpoint *endpoint_new(Port *port);

ProtocolType endpoint_get_type(const Endpoint *endpoint);

const char *endpoint_get_name(const Endpoint *endpoint);

Port *endpoint_get_port(const Endpoint *endpoint);

const char *endpoint_get_ipv4address(const Endpoint *endpoint);
const char *endpoint_get_ipv4netmask(const Endpoint *endpoint);
const char *endpoint_get_ipv4gateway(const Endpoint *endpoint);
const char *endpoint_get_ipv6address(const Endpoint *endpoint);
uint8_t endpoint_get_ipv6prefix(const Endpoint *endpoint);
const char *endpoint_get_ipv6gateway(const Endpoint *endpoint);

void endpoint_free(Endpoint *);

LIST_DECL(Endpoint, endpoint)

#endif
