/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#ifndef NETWORK_PRIVATE_H
#define NETWORK_PRIVATE_H

struct Ports;
struct Connections;

struct Network
{
    /** Number of references to this structure, free if zero */
    int ref_count;
    /** Pointer to backend specific private data */
    void *priv;
    pthread_mutex_t mutex;
    pthread_t thread;
    struct Ports *ports;
    struct Connections *connections;
    struct ActiveConnections *activeConnections;
    GMainLoop *thread_loop;
    volatile int loaded;
};

enum {
    /** Network devices has been loaded, wait if not */
    NETWORK_DEVICES_LOADED = 0x01,
    /** Network settings has been loaded, wait if not */
    NETWORK_SETTINGS_LOADED = 0x02
};

#define MUTEX_LOCK(network) pthread_mutex_lock(&network->mutex)
#define MUTEX_UNLOCK(network) pthread_mutex_unlock(&network->mutex)

void *network_priv_new(Network *);
void network_priv_free(void *);

void network_priv_get_devices(Network *network);
void network_priv_get_active_connections(Network *network);

int network_priv_activate_connection(Network *network, const Port *port, const Connection *connection);

int network_priv_create_connection(Network *network, Connection *connection);
int network_priv_delete_connection(Network *network, Connection *connection);

#endif
