/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#ifndef SETTING_H
#define SETTING_H

#include "globals.h"

Setting *setting_new(ProtocolType protocol_type, SettingType setting_type);
Setting *setting_new_init(ProtocolType protocol_type, SettingType setting_type, const char *address, uint8_t prefix, const char *default_gateway);

SettingType setting_get_type(const Setting *setting);
ProtocolType setting_get_protocol_type(const Setting *setting);
const char *setting_get_id(const Setting *setting);
const char *setting_get_caption(const Setting *setting);

// IP
const char *setting_get_address(const Setting *setting);
uint8_t setting_get_prefix(const Setting *setting);
const char *setting_get_default_gateway(const Setting *setting);

// DNS
void setting_add_dns_server(Setting *setting, const char *dns_server);
size_t setting_get_dns_servers_length(Setting *setting);
const char *setting_get_dns_server(Setting *setting, size_t index);
void setting_add_search_domain(Setting *setting, const char *search_domain);
size_t setting_get_search_domains_length(Setting *setting);
const char *setting_get_search_domain(Setting *setting, size_t index);

// DHCP
const char *setting_get_clientID(const Setting *setting);
void setting_set_clientID(Setting *setting, const char *clientID);

void setting_free(Setting *setting);

LIST_DECL(Setting, setting)

#endif
