/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "setting_nm.h"

#include "connection.h"

#include "setting_private.h"
#include "connection_private.h"

Setting *setting_new_link_local(Connection *connection, enum ProtocolType protocol)
{
    Setting *setting = setting_new(protocol, SETTING_LINK_LOCAL);
    asprintf(&setting->id, "%s_ipv%d", connection->id, (protocol == IPv4) ? 4 : 6);
    asprintf(&setting->caption, "%s IPv%d", connection->name, (protocol == IPv4) ? 4 : 6);
    return setting;
}

Setting *setting_new_DHCP(Connection *connection, enum ProtocolType protocol)
{
    Setting *setting = setting_new(protocol, SETTING_DHCP);
    asprintf(&setting->id, "%s_ipv%d_dhcp", connection->id, (protocol == IPv4) ? 4 : 6);
    asprintf(&setting->caption, "%s IPv%d DHCP", connection->name, (protocol == IPv4) ? 4 : 6);
    return setting;
}

Setting *setting_new_from_static_ipv4(Connection *connection, guint32 address, guint32 prefix, guint32 gateway, int index)
{
    Setting *setting = setting_new(IPv4, SETTING_STATIC);
    asprintf(&setting->id, "%s_ipv4_%d", connection->id, index);
    asprintf(&setting->caption, "%s IPv4 %d", connection->name, index);
    setting->address = ip4ToString(address);
    setting->prefix = prefix;
    setting->default_gateway = ip4ToString(gateway);
    return setting;
}

Setting *setting_new_from_static_ipv6(Connection *connection, GByteArray *address, guint32 prefix, GByteArray *gateway, int index)
{
    Setting *setting = setting_new(IPv6, SETTING_STATIC);
    asprintf(&setting->id, "%s_ipv6_%d", connection->id, index);
    asprintf(&setting->caption, "%s IPv6 %d", connection->name, index);
    setting->address = ip6ToString((struct in6_addr *) address->data);
    setting->prefix = prefix;
    setting->default_gateway = ip6ToString((struct in6_addr *) gateway->data);
    return setting;
}

/*
const char *setting_get_nm_method(Setting *setting)
{
    switch (setting->type) {
        case SETTING_STATIC:
            return (setting->protocol_type == IPv4) ?
                    NM_SETTING_IP4_CONFIG_METHOD_MANUAL :
                    NM_SETTING_IP6_CONFIG_METHOD_MANUAL;
            break;
        case SETTING_DHCP:
            return (setting->protocol_type == IPv4) ?
                    NM_SETTING_IP4_CONFIG_METHOD_AUTO :
                    NM_SETTING_IP6_CONFIG_METHOD_AUTO;
            break;
        case SETTING_LINK_LOCAL:
            return (setting->protocol_type == IPv4) ?
                    NM_SETTING_IP4_CONFIG_METHOD_LINK_LOCAL :
                    NM_SETTING_IP6_CONFIG_METHOD_LINK_LOCAL;
            break;
        default:
            return (setting->protocol_type == IPv4) ?
                    NM_SETTING_IP4_CONFIG_METHOD_DISABLED :
                    NM_SETTING_IP6_CONFIG_METHOD_IGNORE;
    }
}
*/
